/*
  Functions for the ARM7 to process the commands from the ARM9.Based
  on code from the MOD player example posted to the GBADEV forums.
*/
#include <nds.h>
#include "../../common/include/sound_commands.h"
#include "microphone7.h"

static void CommandPlayLoopSample(PlaySampleSoundCommand* ps)
{
  int channel = ps->channel;

  SCHANNEL_CR(channel) = 0; 
  SCHANNEL_TIMER(channel) = SOUND_FREQ(ps->frequency);
  SCHANNEL_SOURCE(channel) = (uint32)ps->data;
  SCHANNEL_LENGTH(channel) = ps->length >> 2;  
  SCHANNEL_REPEAT_POINT(channel) = 0;  
  SCHANNEL_CR(channel) = 
    SCHANNEL_ENABLE | 
    SOUND_REPEAT | 
    SOUND_16BIT | 
    //SOUND_8BIT | 
    SOUND_VOL(ps->volume) | SOUND_PAN(0);
}

static void CommandPlayOneShotSample(PlaySampleSoundCommand* ps)
{
  int channel = ps->channel;

  SCHANNEL_CR(channel) = 0; 
  SCHANNEL_TIMER(channel) = SOUND_FREQ(ps->frequency);
  SCHANNEL_SOURCE(channel) = (uint32)ps->data;
  SCHANNEL_LENGTH(channel) = ps->length >> 2;  
  SCHANNEL_CR(channel) = 
    SCHANNEL_ENABLE | 
    SOUND_ONE_SHOT | 
    //SOUND_8BIT | 
    SOUND_16BIT | 
    SOUND_VOL(ps->volume) | SOUND_PAN(0);
}

static void CommandStopPlaySample(StopPlaySoundCommand* sp)
{
  SCHANNEL_CR(sp->channel) = 0; 
}

/*
 * Start recording
 */
static void CommandStartRecording(StartRecordingCommand* sr)
{
  MicStartRecording(sr->gain, sr->mode, sr->rate, sr->buffer, sr->length);
  commandControl->return_data = 0;
}

/*
 * Start recording
 */
static void CommandRestartRecording(RestartRecordingCommand* sr)
{
  MicRestartRecording(sr->buffer, sr->length);
  commandControl->return_data = 0;
}

/*
 * Stop recording
 */
static void CommandStopRecording()
{
  commandControl->return_data = MicStopRecording();
}

/*
 * Pause recording
 */
static void CommandPauseRecording()
{
  commandControl->return_data = MicPauseRecording();
}

/*
 * Process sound command from ARM9
 */
void CommandProcessCommands() 
{
  static int currentCommand = -1;
  
  while(currentCommand != commandControl->currentCommand) 
  {
    SoundCommand* command = &commandControl->command[currentCommand];

    switch(command->commandType) 
    {
      case PLAY_ONE_SHOT_SAMPLE:
        CommandPlayOneShotSample(&command->playSample);
        break;   
      case PLAY_LOOP_SAMPLE:
        CommandPlayLoopSample(&command->playSample);
        break;   
      case STOP_PLAY:
        CommandStopPlaySample(&command->stopPlay);
        break;
      case START_RECORDING:
        CommandStartRecording(&command->startRecording);
        break;
      case STOP_RECORDING:
        CommandStopRecording();
        break;
      case RESTART_RECORDING:
        CommandRestartRecording(&command->restartRecording);
        break;
      case PAUSE_RECORDING:
        CommandPauseRecording();
        break;
    }
    
    currentCommand++;
    currentCommand &= MAX_COMMANDS-1;
  }
}
