
/* ButtonsHandler declaration

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef BUTTONSHANDLERH
#define BUTTONSHANDLERH

#include "TimersHandler.h"
#include "GraphicsHandler.h"

class Button
{
friend class ButtonsHandler;

protected:

  /*
   * Constructor
   */
  Button(int id, int x, int y, int w, int h, bool bTransparent, bool bGraphic, char* txt, Bitmap* bmp, GraphicsHandler* gh);
  
  /*
   * Destructor
   */
  ~Button();
  
public:

  /*
   * Return the button id
   */
  int GetId() { return (m_id); };
  
  /*
   * Get X position for the button
   */
  int GetX() { return (m_x); };
  
  /*
   * Get Y position for the button
   */
  int GetY() { return (m_y); };
  
  /*
   * Get width for the button
   */
  int GetW() { return (m_w); };
  
  /*
   * Get height for the button
   */
  int GetH() { return (m_h); };
  
  /*
   * Check if button is enabled 
   */
  bool IsEnabled() { return (m_enabled); };
  
  /*
   * Push the button
   */
  void Push();

  /*
   * Release the button
   */
  void Release();
  
  /*
   * Disable the button
   */
  void Disable();
  
  /*
   * Enable the button
   */
  void Enable();

private:

  /*
   * Draw the button
   */
  void Draw();
  
  /*
   * Draw the button text
   */
  void DrawText();
  
  /*
   * Draw the button bitmap
   */
  void DrawBitmap();
  
  /*
   * Draw the button border
   */
  void DrawBorder(bool bClicked);
  
  /*
   * Restore the background
   */
  void RestoreBackground();
  
  /*
   * Button identifier
   */
  int m_id;
  
  /*
   * Button position
   */
  int m_x, m_y;
  
  /*
   * Button size
   */
  int m_w, m_h;
  
  /*
   * Button type
   */
  bool m_bTransparent, m_bGraphic;
  
  /*
   * Button text
   */
  char *m_txt;
  
  /*
   * Button image
   */
  Bitmap* m_bmp;
  
  /*
   * Button enabled
   */
  bool m_enabled;
  
  /*
   * Graphics handler reference
   */
  GraphicsHandler* m_gh;
};

#define MAXBUTTONS    20

class ButtonsHandler
{
public:

  /*
   * Constructor
   */
  ButtonsHandler(GraphicsHandler* gh);
  
  /*
   * Destructor
   */
  ~ButtonsHandler();
  
  /*
   * Add the requested button
   */
  void AddButton(int id, int x, int y, int w, int h, char* txt);
  
  /*
   * Add the requested graphic button
   */
  void AddButton(int id, int x, int y, int w, int h, Bitmap* bmp);
  
  /*
   * Add the requested graphic button area
   */
  void AddButtonArea(int id, int x, int y, int w, int h, Bitmap* bmp);
  
  /*
   * Remove the requested button
   */
  void RemoveButton(int id);
  
  /*
   * Check for button presence at given position
   */
  Button* CheckButtonAt(int x, int y);
  
  /*
   * Retrieve the requested button
   */
  Button* GetButton(int id);
  
  /*
   * Delete all buttons
   */
  void Reset();
  
private:

  /*
   * Add the requested button
   */
  void AddButton(int id, int x, int y, int w, int h, bool bTransparent, bool bGraphic, char*txt, Bitmap* bmp);
  
  /*
   * Buttons defined
   */
  Button* m_buttons[MAXBUTTONS];
  
  /*
   * Graphics handler reference
   */
  GraphicsHandler* m_gh;
};

#ifdef BUTTONSDATA

#define BTN_BACK          RGB15(25, 25, 25)
#define BTN_TEXT          RGB15(0, 0, 0)
#define BTN_DSBTEXT       RGB15(18, 18, 18)
#define BTN_BORDER        RGB15(5, 5, 5)
#define BTN_BORDER_CLICK  RGB15(16, 16, 20)

#endif

#endif
