
/* CombosHandler declaration 

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef COMBOSHANDLERH
#define COMBOSHANDLERH

#include "TimersHandler.h"
#include "GraphicsHandler.h"
#include "Buttonshandler.h"

#define NODEP   -1

struct ComboData
{
  int id;
  int dep;
  int nlit;
};


class Combo
{
friend class CombosHandler;

protected:

  /*
   * Constructor
   */
  Combo(int id, int x, int y, int w, int h, ComboData* data, int dep, int defaultId, GraphicsHandler* gh, ButtonsHandler* bh);
  
  /*
   * Destructor
   */
  ~Combo();
  
public:

  /*
   * Return the combo id
   */
  int GetId() { return (m_id); };
  
  /*
   * Return current element's id
   */
  int GetCurElementId() { return (m_data[m_curData].id); };

  /*
   * Reload data
   */
  void Reload(ComboData* data, int dep, int defaultId);
  
  /*
   * Chek for previous element
   */
  bool HasPrev();
  
  /*
   * Go to previous combo's element
   */
  void Prev();
  
  /*
   * Check for next element
   */
  bool HasNext();
  
  /*
   * Go to next combo's element
   */
  void Next();
  
protected:

  /*
   * Check for previous button at given position
   */
  bool CheckButtonPrevAt(int x, int y); 
 
  /*
   * Check for next button at given position
   */
  bool CheckButtonNextAt(int x, int y); 
 
private:

  /*
   * Draw combo's current element
   */
  void DrawCurElement();
  
  /*
   * Restore the background
   */
  void RestoreBackground();
  
  /*
   * Combo identifier
   */
  int m_id;
  
  /*
   * Previous button
   */
  Button* m_prevButton;
  
  /*
   * Next button
   */
  Button* m_nextButton;
  
  /*
   * Combo position
   */
  int m_x, m_y;
  
  /*
   * Combo size
   */
  int m_w, m_h;
  
  /*
   * Combo data
   */
  ComboData* m_data;
  
  /*
   * Combo data filter
   */
  int m_dep;
  
  /*
   * Combo current data
   */
  int m_curData;
  
  /*
   * Graphics handler reference
   */
  GraphicsHandler* m_gh;
  
  /*
   * Buttons handler reference
   */
  ButtonsHandler* m_bh;
};

#define MAXCOMBOS    10

class CombosHandler
{
public:

  /*
   * Constructor
   */
  CombosHandler(GraphicsHandler* gh, ButtonsHandler* bh);
  
  /*
   * Destructor
   */
  ~CombosHandler();
  
  /*
   * Add the requested combo
   */
  void AddCombo(int id, int x, int y, int w, int h, ComboData* data, int dep, int defaultId);
  
  /*
   * Reload the requested combo
   */
  void ReloadCombo(int id, ComboData* data, int dep, int defaultId);
  
  /*
   * Remove the requested combo
   */
  void RemoveCombo(int id);
  
  /*
   * Check for combo's previous button at given position
   */
  Combo* CheckComboPrevAt(int x, int y);
  
  /*
   * Check for combo's next button at given position
   */
  Combo* CheckComboNextAt(int x, int y);
  
  /*
   * Retrieve the requested combo
   */
  Combo* GetCombo(int id);
  
  /*
   * Delete all combos
   */
  void Reset();
  
private:

  /*
   * Combos defined
   */
  Combo* m_combos[MAXCOMBOS];
  
  /*
   * Graphics handler reference
   */
  GraphicsHandler* m_gh;
  
  /* 
   * Buttons handler reference
   */
  ButtonsHandler* m_bh;
};

#ifdef COMBOSDATA

#define CMB_BACK          RGB15(31, 31, 31)
#define CMB_TEXT          RGB15(0, 0, 0)
#define BTN_BORDER        RGB15(5, 5, 5)
#define BTN_BORDER_CLICK  RGB15(16, 16, 20)

#endif

#endif
