
/* GraphicsHandler header file 

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef GRAPHICS2DH
#define GRAPHICS2DH

struct Bitmap
{
  short w;
  short h;
  unsigned short* bmpData;
  bool btrColor;
  unsigned short trColor;
};

class GraphicsHandler
{
public:

  /*
   * Constructor
   */
  GraphicsHandler();

  /*
   * Clear screen
   */
  void ClearScreen() { ClearScreen(0); };

  /*
   * Clear screen
   */
  void ClearScreen(unsigned short color);

  /*
   * Draw a point
   */
  void DrawPoint(int x, int y, unsigned short color);

  /*
   * Draw a line
   */
  void DrawLine(int x1, int y1, int x2, int y2, unsigned short color);

  /*
   * Draw a rectangle
   */
  void DrawRectangle(int x1, int y1, int x2, int y2, unsigned short color, bool bFill);

  /*
   * Draw a character
   */
  void DrawChar(char c, int x, int y, unsigned short color, bool bDrawBackground, unsigned short bkColor);

  /* 
   * Draw a text
   */
  void DrawText(char *txt, int x, int y, unsigned short color, bool bDrawBackground, unsigned short bkColor);

  /*
   * Get text height
   */
  int GetTextHeight();
  
  /*
   * Get text width 
   */
  int GetTextWidth(char *txt);
  
  /*
   * TrueType fonts
   */
  enum TrueTypeFonts
  {
    TTF_dinobl = 0,
    TTF_goulong,
  };
  
  /* 
   * Draw a text using true type font
   */
  int DrawTextTrueType(TrueTypeFonts font, char *txt, int x, int y, int h, unsigned short color, unsigned short bkColor);
  
  /*
   * Get text width
   */
  int GetTextTrueTypeWidth(TrueTypeFonts font, char *txt, int h);
  
  /*
   * Draw bitmap
   */
  void DrawBitmap(int x, int y, Bitmap* bmp);
  
  /*
   * Make a bitmap from include data
   */
  static void MakeBitmap(Bitmap* bmp, const u8* data, bool btrColor, unsigned short trColor);
  
  void Debug(char *txt);
  
protected:

  /* 
   * Draw a text using true type font
   */
  int DrawTextTrueType(TrueTypeFonts font, char *txt, int x, int y, int h, unsigned short color, unsigned short bkColor, bool bMeasureOnly);
  
  /*
   * Draw a char bitmap for a TrueType text
   */
  void DrawTrueTypeBitmap(int x, int y, u8* bitmap, int bitmapWidth, int bitmapRows, unsigned short color, unsigned short bkColor);

  /*
   * Video memory pointer
   */
  u16* m_vmem;
  
  int yDebug;
    
};

/*
 * Graphics handler for main display
 */
class MainGraphicsHandler : public GraphicsHandler
{
public:
  /*
   * Contructor
   */
  MainGraphicsHandler();  
};

/*
 * Graphics handler for sub display
 */
class SubGraphicsHandler : public GraphicsHandler
{
public:
  /*
   * Constructor
   */
  SubGraphicsHandler();
};

#define UPARROW     0x1E
#define DNARROW     0x1F

#ifdef FONTDATA

#define CHAR_WIDTH    8     // width for a single char
#define CHAR_HEIGHT   12    // height for a single char

#define NBYTESCHAR  12

#define FIRSTCHAR   0x1E
#define LASTCHAR    0x7F

static unsigned char fontData[] = 
{
  0x00, 0x00, 0x00, 0x18, 0x18, 0x3C, 0x3C, 0x7E, 0x7E, 0x00, 0x00, 0x00, // 0x1E up arrow 
  0x00, 0x00, 0x00, 0x7E, 0x7E, 0x3C, 0x3C, 0x18, 0x18, 0x00, 0x00, 0x00, // 0x1F down arrow 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x20 - 
  0x00, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, 0x18, 0x18, 0x00, 0x00, // 0x21 - !
  0x66, 0x66, 0x66, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x22 - "
  0x00, 0x00, 0x66, 0xFF, 0xFF, 0x66, 0x66, 0xFF, 0xFF, 0x66, 0x00, 0x00, // 0x23 - #
  0x00, 0x00, 0x18, 0x7F, 0x98, 0xFE, 0x7E, 0x19, 0xFE, 0x18, 0x00, 0x00, // 0x24 - $
  0x00, 0x00, 0xC3, 0xC7, 0x0E, 0x1C, 0x38, 0x70, 0xE3, 0xC3, 0x00, 0x00, // 0x25 - %
  0x00, 0x00, 0x38, 0x7C, 0x6C, 0x38, 0xF9, 0xCF, 0xFF, 0x7D, 0x00, 0x00, // 0x26 - &
  0x18, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x27 - '
  0x00, 0x0C, 0x1C, 0x38, 0x30, 0x30, 0x30, 0x30, 0x38, 0x1C, 0x0C, 0x00, // 0x28 - (
  0x00, 0x30, 0x38, 0x1C, 0x0C, 0x0C, 0x0C, 0x0C, 0x1C, 0x38, 0x30, 0x00, // 0x29 - )
  0x00, 0x00, 0x00, 0x99, 0xDB, 0x3C, 0x3C, 0xDB, 0x99, 0x00, 0x00, 0x00, // 0x2A - *
  0x00, 0x00, 0x00, 0x18, 0x18, 0x3E, 0x3E, 0x18, 0x18, 0x00, 0x00, 0x00, // 0x2B - +
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x30, 0x00, // 0x2C - ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x3E, 0x3E, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x2D - -
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, // 0x2E - .
  0x00, 0x00, 0x03, 0x07, 0x0E, 0x1C, 0x38, 0x70, 0xE0, 0xC0, 0x00, 0x00, // 0x2F - /
  0x00, 0x00, 0x7E, 0xFF, 0xCF, 0xDF, 0xFB, 0xF3, 0xFF, 0x7E, 0x00, 0x00, // 0x30 - 0
  0x00, 0x00, 0x1C, 0x3C, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C, 0x7E, 0x00, 0x00, // 0x31 - 1
  0x00, 0x00, 0x7E, 0xFF, 0xC7, 0x1E, 0x3C, 0x70, 0xFF, 0xFF, 0x00, 0x00, // 0x32 - 2
  0x00, 0x00, 0xFF, 0xFF, 0x07, 0x1E, 0x1F, 0xC7, 0xFF, 0x7E, 0x00, 0x00, // 0x33 - 3
  0x00, 0x00, 0xC6, 0xC6, 0xC6, 0xFF, 0xFF, 0x06, 0x06, 0x06, 0x00, 0x00, // 0x34 - 4
  0x00, 0x00, 0xFF, 0xFF, 0xC0, 0xFE, 0xFF, 0x03, 0xFF, 0xFE, 0x00, 0x00, // 0x35 - 5
  0x00, 0x00, 0x7F, 0xFF, 0xC0, 0xFE, 0xFF, 0xC3, 0xFF, 0x7E, 0x00, 0x00, // 0x36 - 6
  0x00, 0x00, 0xFF, 0xFF, 0x07, 0x0E, 0x1C, 0x18, 0x18, 0x18, 0x00, 0x00, // 0x37 - 7
  0x00, 0x00, 0x7E, 0xFF, 0xC3, 0x7E, 0xFF, 0xC3, 0xFF, 0x7E, 0x00, 0x00, // 0x38 - 8
  0x00, 0x00, 0x7E, 0xFF, 0xC3, 0xFF, 0x7F, 0x03, 0xFF, 0xFE, 0x00, 0x00, // 0x39 - 9
  0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00, // 0x3A - :
  0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x18, 0x18, 0x30, 0x00, 0x00, // 0x3B - ;
  0x00, 0x00, 0x0E, 0x1C, 0x38, 0x70, 0x70, 0x38, 0x1C, 0x0E, 0x00, 0x00, // 0x3C - <
  0x00, 0x00, 0x00, 0x7E, 0x7E, 0x00, 0x00, 0x7E, 0x7E, 0x00, 0x00, 0x00, // 0x3D - =
  0x00, 0x00, 0xE0, 0x38, 0x1C, 0x0E, 0x0E, 0x1C, 0x38, 0xE0, 0x00, 0x00, // 0x3E - >
  0x00, 0x00, 0xFE, 0xFF, 0x03, 0x1E, 0x18, 0x00, 0x18, 0x18, 0x00, 0x00, // 0x3F - ?
  0x00, 0x00, 0x7E, 0xC7, 0x9F, 0xB3, 0xB3, 0x9F, 0xC0, 0x7F, 0x00, 0x00, // 0x40 - @
  0x00, 0x00, 0x7E, 0xFF, 0xC3, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0x00, 0x00, // 0x41 - A
  0x00, 0x00, 0xFE, 0xFF, 0xC3, 0xFE, 0xFF, 0xC3, 0xFF, 0xFE, 0x00, 0x00, // 0x42 - B
  0x00, 0x00, 0x7F, 0xFF, 0xC0, 0xC0, 0xC0, 0xC0, 0xFF, 0x7F, 0x00, 0x00, // 0x43 - C
  0x00, 0x00, 0xFE, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFE, 0x00, 0x00, // 0x44 - D
  0x00, 0x00, 0xFF, 0xFF, 0xC0, 0xFE, 0xFE, 0xC0, 0xFF, 0xFF, 0x00, 0x00, // 0x45 - E
  0x00, 0x00, 0xFF, 0xFF, 0xC0, 0xFE, 0xFE, 0xC0, 0xC0, 0xC0, 0x00, 0x00, // 0x46 - F 
  0x00, 0x00, 0x7F, 0xFF, 0xC0, 0xCF, 0xCF, 0xC3, 0xFF, 0x7F, 0x00, 0x00, // 0x47 - G
  0x00, 0x00, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0x00, 0x00, // 0x48 - H
  0x00, 0x00, 0x3C, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x3C, 0x00, 0x00, // 0x49 - I
  0x00, 0x00, 0x0F, 0x06, 0x06, 0x06, 0x06, 0xC6, 0xFE, 0x7C, 0x00, 0x00, // 0x4A - J
  0x00, 0x00, 0xC3, 0xC6, 0xCC, 0xF8, 0xF8, 0xCC, 0xC6, 0xC3, 0x00, 0x00, // 0x4B - K
  0x00, 0x00, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xFF, 0xFF, 0x00, 0x00, // 0x4C - L
  0x00, 0x00, 0xC3, 0xE7, 0xFF, 0xDB, 0xC3, 0xC3, 0xC3, 0xC3, 0x00, 0x00, // 0x4D - M
  0x00, 0x00, 0xC3, 0xE3, 0xF3, 0xFB, 0xDF, 0xCF, 0xC7, 0xC3, 0x00, 0x00, // 0x4E - N
  0x00, 0x00, 0x7E, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0x7E, 0x00, 0x00, // 0x4F - O
  0x00, 0x00, 0xFE, 0xFF, 0xC3, 0xFF, 0xFE, 0xC0, 0xC0, 0xC0, 0x00, 0x00, // 0x50 - P
  0x00, 0x00, 0x7E, 0xFF, 0xC3, 0xDB, 0xDF, 0xCF, 0xFF, 0x7F, 0x00, 0x00, // 0x51 - Q
  0x00, 0x00, 0xFE, 0xFF, 0xC3, 0xFF, 0xFE, 0xCC, 0xC6, 0xC3, 0x00, 0x00, // 0x52 - R
  0x00, 0x00, 0x7F, 0xFF, 0xC0, 0xFE, 0x7F, 0x03, 0xFF, 0xFE, 0x00, 0x00, // 0x53 - S
  0x00, 0x00, 0xFF, 0xFF, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, 0x00, // 0x54 - T
  0x00, 0x00, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0x7E, 0x00, 0x00, // 0x55 - U
  0x00, 0x00, 0xC3, 0xC3, 0x66, 0x66, 0x3C, 0x3C, 0x18, 0x18, 0x00, 0x00, // 0x56 - V
  0x00, 0x00, 0xC3, 0xC3, 0xC3, 0xDB, 0xDB, 0xFF, 0xFF, 0x66, 0x00, 0x00, // 0x57 - W
  0x00, 0x00, 0xC3, 0xE7, 0x7E, 0x3C, 0x3C, 0x7E, 0xE7, 0xC3, 0x00, 0x00, // 0x58 - X
  0x00, 0x00, 0xC3, 0xE7, 0x7E, 0x3C, 0x18, 0x18, 0x18, 0x18, 0x00, 0x00, // 0x59 - Y
  0x00, 0x00, 0xFF, 0xFF, 0x0E, 0x1C, 0x38, 0x70, 0xFF, 0xFF, 0x00, 0x00, // 0x5A - Z
  0x00, 0x0F, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0F, 0x00, // 0x5B - [
  0x00, 0x00, 0xC0, 0xE0, 0x70, 0x38, 0x1C, 0x0E, 0x07, 0x03, 0x00, 0x00, // 0x5C - back slash
  0x00, 0xF0, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0xF0, 0x00, // 0x5D - ]
  0x18, 0x3C, 0x66, 0xC3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x5E - ^
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, // 0x5F - _
  0xC0, 0x60, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x60 - '
  0x00, 0x00, 0x00, 0x00, 0x7F, 0xFF, 0xC3, 0xC3, 0xFF, 0x7F, 0x00, 0x00, // 0x61 - a
  0x00, 0x00, 0xC0, 0xC0, 0xFE, 0xFF, 0xC3, 0xC3, 0xFF, 0xFE, 0x00, 0x00, // 0x62 - b
  0x00, 0x00, 0x00, 0x00, 0x7F, 0xFF, 0xC0, 0xC0, 0xFF, 0x7F, 0x00, 0x00, // 0x63 - c
  0x00, 0x00, 0x03, 0x03, 0x7F, 0xFF, 0xC3, 0xC3, 0xFF, 0x7F, 0x00, 0x00, // 0x64 - d
  0x00, 0x00, 0x00, 0x00, 0x7E, 0xE7, 0xFF, 0xE0, 0xFF, 0x7F, 0x00, 0x00, // 0x65 - e
  0x00, 0x00, 0x0C, 0x1C, 0x18, 0x3C, 0x18, 0x18, 0x18, 0x18, 0x00, 0x00, // 0x66 - f
  0x00, 0x00, 0x00, 0x00, 0x7F, 0xFF, 0xC3, 0xFF, 0x7F, 0x03, 0xFF, 0xFE, // 0x67 - g
  0x00, 0x00, 0xC0, 0xC0, 0xFE, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0x00, 0x00, // 0x68 - h
  0x00, 0x00, 0x18, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, 0x00, // 0x69 - i
  0x00, 0x00, 0x0C, 0x00, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x7C, 0x78, // 0x6A - j
  0x00, 0x00, 0xC0, 0xC0, 0xC3, 0xC6, 0xFC, 0xFC, 0xC6, 0xC3, 0x00, 0x00, // 0x6B - k
  0x00, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, 0x00, // 0x6C - l
  0x00, 0x00, 0x00, 0x00, 0xE6, 0xFF, 0xDB, 0xDB, 0xDB, 0xDB, 0x00, 0x00, // 0x6D - m
  0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0x00, 0x00, // 0x6E - n
  0x00, 0x00, 0x00, 0x00, 0x7E, 0xFF, 0xC3, 0xC3, 0xFF, 0x7E, 0x00, 0x00, // 0x6F - o
  0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xC3, 0xC3, 0xFF, 0xFE, 0xC0, 0xC0, // 0x70 - p
  0x00, 0x00, 0x00, 0x00, 0x7F, 0xFF, 0xC3, 0xC3, 0xFF, 0x7F, 0x03, 0x03, // 0x71 - q
  0x00, 0x00, 0x00, 0x00, 0x6E, 0x7E, 0x78, 0x60, 0x60, 0x60, 0x00, 0x00, // 0x72 - r
  0x00, 0x00, 0x00, 0x00, 0x7F, 0xE0, 0x7E, 0x07, 0xFF, 0xFE, 0x00, 0x00, // 0x73 - s
  0x00, 0x00, 0x18, 0x18, 0x3C, 0x18, 0x18, 0x18, 0x1C, 0x0C, 0x00, 0x00, // 0x74 - t
  0x00, 0x00, 0x00, 0x00, 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0x7F, 0x00, 0x00, // 0x75 - u
  0x00, 0x00, 0x00, 0x00, 0xC3, 0xC3, 0x66, 0x66, 0x3C, 0x3C, 0x00, 0x00, // 0x76 - v
  0x00, 0x00, 0x00, 0x00, 0xC3, 0xC3, 0xDB, 0xDB, 0xFF, 0x66, 0x00, 0x00, // 0x77 - w
  0x00, 0x00, 0x00, 0x00, 0xC3, 0xE7, 0x7E, 0x7E, 0xE7, 0xC3, 0x00, 0x00, // 0x78 - x
  0x00, 0x00, 0x00, 0x00, 0xC3, 0xC3, 0xC3, 0xFF, 0x7F, 0x03, 0xFF, 0xFE, // 0x79 - y
  0x00, 0x00, 0x00, 0x00, 0xFF, 0xFE, 0x1C, 0x38, 0x7F, 0xFF, 0x00, 0x00, // 0x7A - z
  0x00, 0x03, 0x06, 0x06, 0x06, 0x1C, 0x1C, 0x06, 0x06, 0x06, 0x03, 0x00, // 0x7B - {
  0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, // 0x7C - |
  0x00, 0xC0, 0x60, 0x60, 0x60, 0x3C, 0x3C, 0x60, 0x60, 0x60, 0xC0, 0x00, // 0x7D - }
  0x00, 0x00, 0x00, 0x00, 0x86, 0xDF, 0xFB, 0x61, 0x00, 0x00, 0x00, 0x00, // 0x7E - ~
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x7F -
};

#endif

#endif
