
/* PitchDetecor definition - Derived from qtuneroid-0.9: */

/***************************************************************************
					pitchdetector.h  -  description
						 -------------------
		begin                : Fri Jun 30 2000
		copyright            : (C) 2000 Jozef Kosoru,(C) 2004 Boo-boo
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
 /* Changes:
 
  - porting to DevKitPRO for recompilation
  - various implementation to base algorithm to improve the analisys stability
 
  Roberto Navone <ron@object65.it>
  
 */


#ifndef PITCHDETECTORH
#define PITCHDETECTORH

#include <nds.h>

typedef short SAMPLE;

struct PeakStore
{
  int position;
  SAMPLE value;
};

struct ValueRange
{
  int lo;
  int hi;
};

#define FREQUNDEF   -1.0

class PitchDetector
{
public:

  /*
   * Constructor
   */
  PitchDetector(int sampleRate, SAMPLE threshold);

  /*
   * Destructor
   */
  ~PitchDetector();
    
  /*
   * Analyze
   */
	float analyzeBuffer(SAMPLE *buffer, int bufSize);
  
  /*
   * Get peak
   */
  float getPeak() {return ppeak;}
  
  /*
   * Get level
   */
  float getLevel() {return plevel;}
  
  /*
   * Get analisys quality (1 to 5)
   */
  int getAnalisysQuality() { return aquality; }
  
private:

	enum SearchPhase { lowSignal, topPeak, bottomPeak };
  
  float analyzeBuffer(PeakStore* peakBuff, int peakNum, int start, bool *bBothDirections);

  static int roundToInt(const float f);

	static int averagePeak(const PeakStore* peaksBuf, const int bufSize);

	int giveBasePeriod(int& cStep, const PeakStore* peakBuff, const int start, const int stop, const bool rightDirect) const;
                
	static ValueRange computeRange(const int num, const float difMulti);
                
	int findAnotherPeriod(const int cPeriod, int& bufStep, const PeakStore* peakBuff, const int stop, const bool rightDirect) const;
  
  bool similar(float a, float b);
  
  bool notTooDifferent(float a, float b);

  float m_samplerate;   // signal sample rate
	SAMPLE* m_buffer;     // sample's buffer
	int m_bufSize;        // samples's buffer size
	SAMPLE m_threshold;   // samples under this value are ignored
  float m_maxPeriodDif; // max period difference between two neighbouring valid peaks
  float m_maxAmpDif;    // max amplitude difference between two neighbouring valid peaks
  int m_minPeaksAmount; // min number of valid peaks (2=min!)
	int m_peaksMinimum;   // min number of peaks
  int m_minPeriod, m_maxPeriod; // frequency range
	PeakStore* m_peakBufPos;      // buffer for positive peaks
	PeakStore* m_peakBufNeg;      // buffer for negative peaks
	float ppeak;          //maximum signal level
	float plevel;
	double maxIntValue;
  int aquality;
  float lastf;
  
};

#endif
