
/* Sound handler definition 

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef SOUNDHANDLERH
#define SOUNDHANDLERH

class SoundHandler
{
public:

  /*
   * Constructor
   */
  SoundHandler();
  
  /*
   * Input signal gain
   */
  enum MicGain
  {
    MICGAIN_20 = 0,
    MICGAIN_40,
    MICGAIN_80,
    MICGAIN_160
  };
  
  /*
   * Recording mode 
   */
  enum RecMode
  {
    MICMODE_8BIT = 0,
    MICMODE_12BIT = 1  
  };
  
  /*
   * Start recording
   */
  void StartRecording(MicGain gain, RecMode mode, int rate, void* buffer, int length);
  
  /*
   * Restart recording
   */
  void RestartRecording(void* buffer, int length);
  
  /*
   * Stop recording
   */
  int StopRecording();
  
  /*
   * Pause recording
   */
  int PauseRecording();
  
  /*
   * Play the requested sample
   */
  void PlaySample(float freq, void *sampleData, int lSampleData);
  
  /*
   * Play the requested looped sample
   */
  void PlayLoopSample(float freq, void *sampleData, int lSampleData);
  
  /*
   * Stop play
   */
  void StopPlaySample();
  
  void CommandPlayOneShotSample(int channel, int frequency, void* data, int length, int volume);
  
};

#endif

