
/* Timers handler definition
 
Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef TIMERHANDLERH
#define TIMERHANDLERH
 
extern unsigned long tctr;    // time counter i 1/60th of seconds 

class Timer
{
friend class TimersHandler;

protected:

  /* 
   * Constructor
   */
  Timer(int timeout, bool bAutoRestart);

public:

  /*
   * Check if elapsed
   */
  bool IsElapsed(); 
  
  /*
   * Touch timer
   */
  void Touch();
  
  /*
   * Restart timer
   */
  void Restart();
  
private:
  
  /*
   * Start timer
   */
  void Start();
  
  /*
   * Get the time counter
   */
  unsigned long GetTimeCounter();
  
  /* 
   * Timeout
   */
  unsigned long m_timeout;

  /*
   * Time counter at timeout
   */
  unsigned long m_tctrTimeout;
  
  /*
   * Auto restart flag
   */
  bool m_bAutoRestart;
};

class TimersHandler
{
public:

  /*
   * Create a new timer
   */
  static Timer* CreateTimer(int timeout, bool bAutoRestart);
};

#endif


