
/* ToneCalculator definition - Derived from qtuneroid-0.9: */

/***************************************************************************
					tonedetermine.h  -  description
						 -------------------
		begin                : Sat Jul 8 2000
		copyright            : (C) 2000 Jozef Kosoru,(C) 2004 Boo-boo
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* Changes:

  - porting to DevKitPRO for recompilation
  - added octaves handling
  - added method for check tuning respect a given tone/octave
  
  Roberto Navone <ron@object65.it>

*/

#ifndef TONEDETERMINE_H
#define TONEDETERMINE_H

#include <math.h>

#define TONEUNDEF 999

#define MAXDIVTUNED   15

class ToneCalculator
{
public:
  /*
   * Constructor (rif. frequency of A setted to 440.0)
   */
	ToneCalculator();
  
  /*
   * Destructor
   */
	~ToneCalculator();
  
  /*
   * Get tone correspondent to requested frequency 0=C...11=B
   */
	int getTone(const float freq);
  
  /*
   * Get octave correspondent to requested frequency 0/8
   */
	int getOctave(const float freq);
  
  /*
   * Get requested frequency divergency from tone (-32,0,32)
   */
	int getDivergency(const float freq);
  
  /*
   * Get the frequency correspondent to requested note/octave
   * note from 0 (C) to 11 (B)
   */
  float getFreq(int note, int octave);
  
  /*
   * Set new rif. frequency to A tone
   */
	void setNewFreqOf_A(float freqOf_A);

  /*
   * Check if note is tuned
   */
  bool isTuned(int divergency);
  
  /*
   * Check if note is tuned
   */
  bool isTuned(int note, int octave, int divergency, int targetNote, int targetOctave);
  
private:
	inline void analyzeFreq(const float freq);
	double m_logFreqOf_A;
	float m_lastFreq;
	int m_divergence;
	int m_tone, m_octave;
	double m_logToneFreq[12];
	static const double m_toneIncrease;
	static const double m_log2;

	inline static int roundToInt(const double f)
	{
		const int fi = (int)(f);
		if ((f - (double)(fi)) >= 0.5)
		{
			return fi + 1;
		}
		else
		{
			return fi;
		}
	};

};

#endif
