
/* CombossHandler implementation

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include "stdio.h"

#include "LiteralHandler.h"
#define COMBOSDATA
#include "CombosHandler.h"
#include "GraphicsHandler.h"

static char uparrow[] = { UPARROW, 0, };
static char dnarrow[] = { DNARROW, 0, };

/*
 * Constructor
 */
Combo::Combo(int id, int x, int y, int w, int h, ComboData* data, int dep, int defaultId, GraphicsHandler* gh, ButtonsHandler* bh)
{
  m_id = id;
  m_x = x;
  m_y = y;
  m_w = w;
  m_h = h;
  m_data = data;
  m_dep = dep;
  m_gh = gh;
  m_bh = bh;

  /* Create prev & next buttons */
  
  int idb = id * 100000+1;
  
  bh->AddButton(idb, x, y, h, h, uparrow);
  m_prevButton = bh->GetButton(idb);
  
  bh->AddButton(idb+1, x+w-2*h, y, h, h, dnarrow);
  m_nextButton = bh->GetButton(idb+1);
  
  /* Set on default element */
  
  m_curData = 0;
  
  while (true)
    if (data[m_curData].dep == dep  &&  data[m_curData].id == defaultId)
      break;
    else
      m_curData++;
  
  /* Draw current element */
  
  DrawCurElement();
}

/*
 * Destructor
 */
Combo::~Combo()
{
  m_bh->RemoveButton(m_prevButton->GetId());
  m_bh->RemoveButton(m_nextButton->GetId());
  
  RestoreBackground();
}

/*
 * Reload data
 */
void Combo::Reload(ComboData* data, int dep, int defaultId)
{
  m_data = data;
  m_dep = dep;
  
  /* Set on default element */
  
  m_curData = 0;
  
  while (true)
    if (data[m_curData].dep == dep  &&  data[m_curData].id == defaultId)
      break;
    else
      m_curData++;
  
  /* Draw current element */
  
  DrawCurElement();
}

/*
 * Chek for previous element
 */
bool Combo::HasPrev()
{
  int n = m_curData;
  
  while (--n >= 0)
    if (m_data[n].dep == m_dep)
      return (true);
      
  return (false);
}
  
/*
 * Go to previous combo's element
 */
void Combo::Prev()
{
  int n = m_curData;
  
  while (--n >= 0)
    if (m_data[n].dep == m_dep)
    {
      m_curData = n;
      
      DrawCurElement();
      
      return;
    }
}

/*
 * Check for next element
 */
bool Combo::HasNext()
{
  int n = m_curData;
  
  while (m_data[++n].id != -1)
    if (m_data[n].dep == m_dep)
      return (true);
      
  return (false);
}

/*
 * Go to next combo's element
 */
void Combo::Next()
{
  int n = m_curData;
  
  while (m_data[++n].id != -1)
    if (m_data[n].dep == m_dep)
    {
      m_curData = n;
      
      DrawCurElement();
      
      return;
    }
}

/*
 * Check for previous button at given position
 */
bool Combo::CheckButtonPrevAt(int x, int y)
{
  if (!HasPrev())
    return (false);
   
  if (x >= m_prevButton->GetX()  &&  x <= (m_prevButton->GetX() + m_prevButton->GetW())  &&
      y >= m_prevButton->GetY()  &&  y <= (m_prevButton->GetY() + m_prevButton->GetH()))
    return (true);
    
  return (false);
}

/*
 * Check for next button at given position
 */
bool Combo::CheckButtonNextAt(int x, int y)
{
  if (!HasNext())
    return (false);
    
  if (x >= m_nextButton->GetX()  &&  x <= (m_nextButton->GetX() + m_nextButton->GetW())  &&
      y >= m_nextButton->GetY()  &&  y <= (m_nextButton->GetY() + m_nextButton->GetH()))
    return (true);
    
  return (false);
}

/*
 * Draw the combo
 */
void Combo::DrawCurElement()
{
  m_gh->DrawRectangle(m_x+m_h, m_y, m_x+m_w-2*m_h-1, m_y+m_h, CMB_BACK, true);
  
  int tx, ty;
  
  ty = m_y + ((m_h - m_gh->GetTextHeight()) >> 1);
  tx = m_x+m_h+1;
  
  m_gh->DrawText(LiteralHandler::GetLit(m_data[m_curData].nlit), tx, ty, CMB_TEXT, false, 0);
  
  /* Enable/disable buttons depending on current item */
  
  bool bPrev = HasPrev();
  bool bNext = HasNext();
  
  if (m_prevButton->IsEnabled()  &&  !bPrev)
    m_prevButton->Disable();
  else if (!m_prevButton->IsEnabled()  &&  bPrev)
    m_prevButton->Enable();
    
  if (m_nextButton->IsEnabled()  &&  !bNext)
    m_nextButton->Disable();
  else if (!m_nextButton->IsEnabled()  &&  bNext)
    m_nextButton->Enable();
}

/*
 * Restore the background
 */
void Combo::RestoreBackground()
{
  //m_gh->DrawRectangle(m_x, m_y, m_x + m_w, m_y + m_h, SCR_BACK, true);
}
  
/*
 * Constructor
 */
CombosHandler::CombosHandler(GraphicsHandler* gh, ButtonsHandler* bh)
{
  m_gh = gh;
  m_bh = bh;
  
  for (int i=0; i<MAXCOMBOS; i++)
    m_combos[i] = 0;
}

/*
 * Destructor
 */
CombosHandler::~CombosHandler()
{
  for (int i=0; i<MAXCOMBOS; i++)
    if (m_combos[i] != 0)
      delete m_combos[i];
}

/*
 * Add the requested combo
 */
void CombosHandler::AddCombo(int id, int x, int y, int w, int h, ComboData* data, int dep, int defaultId)
{
  int n = -1;
  
  for (int i=0; i<MAXCOMBOS; i++)
  {
    if (m_combos[i] != 0)
    {
      if (m_combos[i]->GetId() == id)
        return;
    }
    else if (n == -1)
      n = i;
  }
  
  if (n != -1)
    m_combos[n] = new Combo(id, x, y, w, h, data, dep, defaultId, m_gh, m_bh);
}

/*
 * Reload the requested combo
 */
void CombosHandler::ReloadCombo(int id, ComboData* data, int dep, int defaultId)
{
  for (int i=0; i<MAXCOMBOS; i++)
    if (m_combos[i] != 0)
    {
      if (m_combos[i]->GetId() == id)
      {
        m_combos[i]->Reload(data, dep, defaultId);
        
        break;
      }
    }  
}

/*
 * Remove the requested combo
 */
void CombosHandler::RemoveCombo(int id)
{
  for (int i=0; i<MAXCOMBOS; i++)
    if (m_combos[i] != 0)
    {
      if (m_combos[i]->GetId() == id)
      {
        delete m_combos[i];
        
        m_combos[i] = 0;
        
        break;
      }
    }  
}

/*
 * Check for combo's previous button at given position
 */
Combo* CombosHandler::CheckComboPrevAt(int x, int y)
{
  for (int i=0; i<MAXCOMBOS; i++)
    if (m_combos[i] != 0)
    {
      Combo* cmb = m_combos[i];
      
      if (cmb->CheckButtonPrevAt(x, y))
        return (cmb);
    }

  return (0);
}
  
/*
 * Check for combo's next button at given position
 */
Combo* CombosHandler::CheckComboNextAt(int x, int y)
{
  for (int i=0; i<MAXCOMBOS; i++)
    if (m_combos[i] != 0)
    {
      Combo* cmb = m_combos[i];
      
      if (cmb->CheckButtonNextAt(x, y))
        return (cmb);
    }
    
  return (0);
}
  
/*
 * Retrieve the requested combo
 */
Combo* CombosHandler::GetCombo(int id)
{
  for (int i=0; i<MAXCOMBOS; i++)
    if (m_combos[i] != 0)
      if (m_combos[i]->GetId() == id)
        return (m_combos[i]);
    
  return (0);
}

/*
 * Delete all combos
 */
void CombosHandler::Reset()
{
  for (int i=0; i<MAXCOMBOS; i++)
    if (m_combos[i] != 0)
    {
      delete m_combos[i];
      
      m_combos[i] = 0;
    }  
}
