
/* PitchDetector implementation - Derived from qtuneroid-0.9: */

/***************************************************************************
					pitchdetector.cpp  -  description
						 -------------------
		begin                : Fri Jun 30 2000
		copyright            : (C) 2000 Jozef Kosoru,(C) 2004 Boo-boo
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#define TUNEROIDPD
#include "PitchDetector.h"

//#define _DEBUG_PD

/*
 * Constructor
 */
PitchDetector::PitchDetector(int sampleRate, SAMPLE threshold)
{
  int maxFreq = 4200;
  int minFreq = 20;
  float maxFreqDif = 0.2;
  float maxAmpDif = 0.4;  //0.2;
  float calibration = 1.0;
  int minPeaksAmount = 4; //2;
	m_samplerate = (float)sampleRate * calibration;
	//m_buffer = buffer;
	//m_bufSize = bufSize;
	m_threshold = threshold;
	m_maxPeriodDif = maxFreqDif;
	m_maxAmpDif = maxAmpDif;
	m_minPeaksAmount = minPeaksAmount;
	m_minPeriod = roundToInt(sampleRate / (float)maxFreq);
	m_maxPeriod = roundToInt(sampleRate / (float)minFreq);
	//m_peaksMinimum = m_bufSize / m_maxPeriod;
	//m_peakBufPos = (PeakStore*)malloc(bufSize / 4 * sizeof(PeakStore));
	//m_peakBufNeg = (PeakStore*)malloc(bufSize / 4 * sizeof(PeakStore));
  
	maxIntValue = 0;

	switch(sizeof(SAMPLE))
	{
		case 1: maxIntValue = 127.0; break;
		case 2: maxIntValue = 2048.0; break;
		//case 4: maxIntValue = INT32_MAX; break;
		//case 8: maxIntValue = INT64_MAX; break;
	}
  
  lastf = -1;
  
  m_peakBufNeg = 0;
  m_peakBufPos = 0;
}

/*
 * Destructor
 */
PitchDetector::~PitchDetector()
{
  free(m_peakBufNeg);
  free(m_peakBufPos);
}
  
/*
 * Analyze
 */
float PitchDetector::analyzeBuffer(SAMPLE *buffer, int bufSize)
{
  if (m_peakBufNeg != 0)
    free(m_peakBufNeg);
    
  if (m_peakBufPos != 0)
    free(m_peakBufPos);
  
	m_buffer = buffer;
	m_bufSize = bufSize;
  
	m_peaksMinimum = m_bufSize / m_maxPeriod;
	m_peakBufPos = (PeakStore*)malloc(bufSize / 4 * sizeof(PeakStore));
	m_peakBufNeg = (PeakStore*)malloc(bufSize / 4 * sizeof(PeakStore));

	int count;
	SearchPhase status = lowSignal;
	int posPos = 0, negPos = 0;
	bool upCross = false, downCross = false;
	m_peakBufPos[0].value = m_peakBufNeg[0].value = 0;
	int cTopPos = 0, cBottomPos = 0; // defalut values are not needed, but a compiler gives warning
	int cTopPeak = 0, cBottomPeak = 0;

	double esum=0.0,e;	
	ppeak=0.0;
	
	/* Fill peak buffers */
  
	for(count = 0; count < m_bufSize - 1; ++count)
	{	
		e=(((float)labs(m_buffer[count]))/maxIntValue);
    
		esum+=e;
    
		if(e > ppeak) 
      ppeak = e; 
		
		switch(status)
		{
			case lowSignal: // process samples between -threshold & threshold level
				if (m_buffer[count] > m_threshold && m_buffer[count + 1] > m_threshold)
				{
					status = topPeak;
					upCross = true;

					if (downCross)
					{
						if (m_peakBufNeg[negPos].value > cBottomPeak)
						{
							cBottomPeak = m_peakBufNeg[negPos].value;
							cBottomPos = negPos;
						}

						negPos++;
						downCross = false;
						m_peakBufNeg[negPos].value = 0;
					}
				}
				else if (m_buffer[count] < -m_threshold && m_buffer[count+1] < -m_threshold)
				{
					status = bottomPeak;
					downCross = true;
					if (upCross)
					{
						if (m_peakBufPos[posPos].value > cTopPeak)
						{
							cTopPeak = m_peakBufNeg[posPos].value;
							cTopPos = posPos;
						}

						posPos++;
						upCross = false;
						m_peakBufPos[posPos].value = 0;
					}
				}
				break;

			case topPeak: // process samples bigger that threshold level
				if (m_buffer[count] <= m_threshold)
				{
					status = lowSignal;
				}
				else if (m_buffer[count] > m_peakBufPos[posPos].value)
				{
					m_peakBufPos[posPos].value = m_buffer[count];
					m_peakBufPos[posPos].position = count;
				}
				break;

			case bottomPeak: // process samples smaller that -threshold level
				if (-m_buffer[count] < m_threshold)
				{
					status = lowSignal;
				}
				else if (-m_buffer[count] > m_peakBufNeg[negPos].value)
				{
					m_peakBufNeg[negPos].value = -m_buffer[count];
					m_peakBufNeg[negPos].position = count;
				}
				break;
		}
	}
	
	plevel=esum/count; 
	
	/* There must be at least m_peaksMinimum samples on the positive or negative side */
  
	if (posPos < m_peaksMinimum && negPos < m_peaksMinimum)
	{
#ifdef _DEBUG_PD
    /*
		printf("unsufficient amount of peaks\n");
		printf("posPos: %d negPos: %d\n", posPos, negPos);
    */
#endif
		return (lastf = -1.0);  // unsufficient amount of peaks
	}

	/* Compute average peak for positive and negative values */
  
	const int averagePos = averagePeak(m_peakBufPos, posPos);
	const int averageNeg = averagePeak(m_peakBufNeg, negPos);

	/* Decide which side (positive/negative) have priority according an average peak */
  
  bool bPosPriority = averagePos >= averageNeg;

  /* Analyze with positive peaks */

  bool posFreqBothDirections; 
  
  float posFreq = analyzeBuffer(m_peakBufPos, posPos, cTopPos, &posFreqBothDirections);
  
  /* Analyze with negative peaks */

  bool negFreqBothDirections; 
  
  float negFreq = analyzeBuffer(m_peakBufNeg, negPos, cBottomPos, &negFreqBothDirections);
  
  /* Work on values to determine the correct frequency */
  
  if (posFreq < 0.0  &&  negFreq < 0.0)
    return (lastf = -1.0);
    
  float freq = -1.0;
  
  if (posFreq > 0.0  &&   negFreq > 0.0)
  {
    if (similar(posFreq, negFreq))
    {
      // both sides similar
      freq = (posFreq + negFreq) / 2.0;
      
      if (lastf > 0.0  &&  !notTooDifferent(freq, lastf))
        freq = -1.0;
      else
        aquality = 1;   // both sides similar
    }
    else if (lastf > 0)
    {
      // sides different, che if one is similar to last freq
      if (similar(posFreq, lastf))
      {
        freq = posFreq;

        aquality = 3 - (bPosPriority ? 1 : 0);  // only one side similar to last freq
      }
      else if (similar(negFreq, lastf))
      {
        freq = negFreq;
        
        aquality = 3 - (bPosPriority ? 0 : 1);  // only one side similar to last freq
      }
    }
    else
    {
      freq = bPosPriority ? posFreq : negFreq;
      
      aquality = 4;   // no match with history
    }
  }
  else if (posFreq > 0.0)
  {
    // only positive sides
    if (lastf > 0.0  && similar(posFreq, lastf))
    {
      freq = posFreq;
      
      aquality = 3 - (bPosPriority ? 1 : 0);  // only one side similar to last freq
    }
    else if (lastf < 0.0)
    {
      freq = posFreq;
      
      aquality = 4;   // no match with history
    }
  }
  else if (negFreq > 0.0)
  {
    // only negative side
    if (lastf > 0.0  &&  similar(negFreq, lastf))
    {
      freq = negFreq;
      
        aquality = 3 - (bPosPriority ? 0 : 1);  // only one side similar to last freq
    }
    else if (lastf < 0.0)
    {
      freq = negFreq;
      
      aquality = 4;   // no match with history
    }
  }
  
  return (lastf = freq);
}


/*
 * Analyze
 */
float PitchDetector::analyzeBuffer(PeakStore* peakBuff, int peakNum, int start, bool *bBothDirections)
{  
	/* Choose process direction */
  
	int stop1, stop2;
	bool rightDirect; //true = --> / false = <--
  
  if (peakNum == start)
    return -2;  // assert(peakNum != start);

	if ((peakNum - start - 1) >= start)
	{	// direction = -->
		stop1 = peakNum;
		stop2 = -1;
		rightDirect = true;
    //printf("rightDirect\n");
	}
	else
	{	// direction = <--
		stop1 = -1;
		stop2 = peakNum;
		rightDirect = false;
    //printf("leftDirect\n");
	}

	// find base period
	int periodsFound = 0, periodsSum = 0, cPeriod, cStep = start;
	// periodsFound and periodsSum don't need defaut values, but a compiler gives warning
  
//  int bestcPeriod = -1, npfBest = -1, psbest = -1;

	while ((cPeriod = giveBasePeriod(cStep, peakBuff, start, stop1, rightDirect)) != 0)
	{
    //printf("cp = %d ", cPeriod);
    
		int aPeriod = cPeriod, bufStep = cStep;
		periodsFound = 1;
		periodsSum = cPeriod;

		while ((aPeriod = findAnotherPeriod(aPeriod, bufStep, peakBuff, stop1, rightDirect)) > 0)
		{
			++periodsFound;
			periodsSum += aPeriod;
    
		}

    if (periodsFound > m_minPeaksAmount)
      break;
      /* Commented because it scale always on lower tones
      if (periodsFound > npfBest)
      {
        bestcPeriod = cPeriod;
        npfBest = periodsFound;
        psbest = periodsSum;
      }
      */

		if(!aPeriod)
		{	// no more peaks (maybe we found the right period)
			break;
		}
		// else aPeriod=-1 -> we must try next basePeriod
	}

  /*
  if (periodsFound < npfBest)
  {
    // ron: restore best result
    
    cPeriod = bestcPeriod;
    periodsFound = npfBest;
    periodsSum = psbest;
  }
  */
  
	if (!cPeriod || periodsFound < m_minPeaksAmount)
	{
#ifdef _DEBUG_PD
    printf("2 %d, %d, %d\n", cPeriod, periodsFound, m_minPeaksAmount);
		//printf("no base period / there was few of suitable peaks\n");
#endif
		return -3;  // no base period / there was few of suitable peaks
	}

	// evaluate other direction
	int aPeriod = cPeriod;
	int bufStep = start;
	rightDirect =! rightDirect;

	while ((aPeriod = findAnotherPeriod(aPeriod, bufStep, peakBuff, stop2, rightDirect)) > 0)
	{
		++periodsFound;
		periodsSum += aPeriod;
	}

	*bBothDirections = aPeriod != -1;
  
	const float resultPeriod = (float)periodsSum / (float)periodsFound;
  
	return m_samplerate / resultPeriod;  // return resultant frequency
}


int PitchDetector::roundToInt(const float f)
{
	const int fi = (int)f;

	if ((f - (float)fi) >= 0.5)
	{
		return fi + 1;
	}
	else
	{
		return fi;
	}
}

int PitchDetector::averagePeak(const PeakStore* peaksBuf, const int bufSize)
{
	if (bufSize)
	{
		int sum = 0;
		for (int count = 0; count < bufSize; ++count)
		{
			sum += peaksBuf[count].value;
		}

		return sum / bufSize;
	}
	else
	{
		return 0;
	}
}

inline int PitchDetector::giveBasePeriod(int& cStep, const PeakStore* peakBuff, const int start, const int stop, const bool rightDirect) const
{
	static int peakValue;
	static ValueRange ampRange;

	if (peakValue != (int)peakBuff[start].value)
	{	// peak value has changed -> compute new range
		peakValue = (int)peakBuff[start].value;
		ampRange = computeRange(peakValue, m_maxAmpDif);
	}

#ifdef _DEBUG_PD
  printf("4 ");
	//printf("ENTER cStep %d\n", cStep);
#endif

	if (rightDirect)
	{
		while ((++cStep) < stop)
		{
			const PeakStore peak = peakBuff[cStep];
			const int periodSize = peak.position-peakBuff[start].position;

			if (periodSize > m_maxPeriod)
			{
				break;
			}
			else if (periodSize < m_minPeriod)
			{
#ifdef _DEBUG_PD
        printf("5 ");
				//printf("TOO SMALL - %d", periodSize);
#endif
				continue;
			}
			else if (peak.value <= ampRange.hi && peak.value >= ampRange.lo)
			{
#ifdef _DEBUG_PD
        printf("6 ");
        /*
				printf(">>TRY----: %d value: %d\n", periodSize, peak.value);
				printf("cStep: %d\n", cStep);
        */
#endif
				return periodSize;
			}

#ifdef _DEBUG_PD
      printf("7 ");
			//printf("baseFAIL: %d ampDif: %d/%d", periodSize, ampRange.hi, ampRange.lo);
#endif
		}
	}
	else
	{
		while ((--cStep) > stop)
		{
			const PeakStore peak = peakBuff[cStep];
			const int periodSize = peakBuff[start].position - peak.position;

			if (periodSize > m_maxPeriod)
			{
				break;
			}
			else if (periodSize < m_minPeriod)
			{
#ifdef _DEBUG_PD
        printf("8 ");
				//printf("TOO SMALL - %d\n", periodSize);
#endif
				continue;
			}
			else if (peak.value <= ampRange.hi && peak.value >= ampRange.lo)
			{
#ifdef _DEBUG_PD
        printf("9 ");
        /*
				printf(">>TRY----: %d value: %d\n", periodSize, peak.value);
				printf("cStep: %d\n", cStep);
        */
#endif
				return periodSize;
			}
#ifdef _DEBUG_PD
      printf("10 ");
			//printf("baseFAIL: %d ampDif: %d/%d", periodSize, ampRange.hi, ampRange.lo);
#endif
		}
	}
	return 0; // no suitable peak
}

ValueRange PitchDetector::computeRange(const int num, const float difMulti)
{
	const int dif = roundToInt((float)num * difMulti);
	const ValueRange range = {num - dif, num + dif};
	return range;
}

int PitchDetector::findAnotherPeriod(const int cPeriod, int& bufStep, const PeakStore* peakBuff, const int stop, const bool rightDirect) const
{
	const PeakStore prevPeak = peakBuff[bufStep];
	const ValueRange ampRange = computeRange((int)prevPeak.value, m_maxAmpDif);
	const ValueRange periodRange = computeRange(cPeriod, m_maxPeriodDif);
#ifdef _DEBUG_PD
  printf("B");
	//printf("find__bufStep: %d\n", bufStep);
#endif

	if (rightDirect)
	{
		while ((++bufStep) < stop)
		{
			const PeakStore peak = peakBuff[bufStep];
			const int periodSize = peak.position - prevPeak.position;

			if (periodSize > periodRange.hi || periodSize > m_maxPeriod || periodSize < m_minPeriod)
			{
#ifdef _DEBUG_PD
        printf("C");
        /*
        printf("periodFAIL: %d pRange: %d|%d", periodSize, periodRange.hi, periodRange.lo);
				printf("__bufStep: %d PEAK: %d\n", bufStep, peak.position);
        */
#endif
				return -1;
			}
			else if (peak.value <= ampRange.hi && peak.value >= ampRange.lo &&
					 periodSize >= periodRange.lo)
			{
				if ((bufStep + 1) < stop)
				{
					int nextPeriodSize = peakBuff[bufStep + 1].position - prevPeak.position;
					if (abs(cPeriod - nextPeriodSize) < abs(cPeriod - periodSize))
					{
						continue; // next peak will be better
					}
				}
				return periodSize;  // this was suitable peak
			}

#ifdef _DEBUG_PD
      printf("D");
			//printf("FAIL: %d ampDif: %d/%d periodDif: %d/%d\n", periodSize, ampRange.hi, ampRange.lo, periodRange.hi, periodRange.lo);
#endif
		}
	}
	else
	{
		while ((--bufStep) > stop)
		{
			const PeakStore peak = peakBuff[bufStep];
			const int periodSize = prevPeak.position - peak.position;
			if (periodSize > periodRange.hi || periodSize > m_maxPeriod ||
				periodSize < m_minPeriod)
			{
#ifdef _DEBUG_PD
        printf("E");
        /*
        printf("periodFAIL: %d pRange: %d|%d", periodSize, periodRange.hi, periodRange.lo);
				printf("__bufStep: %d PEAK: %d\n", bufStep, peak.position);
        */
#endif
				return -1;
			}
			else if (peak.value <= ampRange.hi && peak.value >= ampRange.lo &&
					 periodSize >= periodRange.lo)
			{
				if ((bufStep-1) > stop)
				{
					int nextPeriodSize = prevPeak.position - peakBuff[bufStep-1].position;
					if(abs(cPeriod - nextPeriodSize) < abs(cPeriod - periodSize))
					{
						continue; // next peak will be better
					}
				}
				return periodSize;  // this was suitable peak
			}

#ifdef _DEBUG_PD
      printf("F");
			//printf("FAIL: %d ampDif: %d/%d periodDif: %d/%d\n", periodSize, ampRange.hi, ampRange.lo, periodRange.hi, periodRange.lo);
#endif
		}
	}

	return 0; // no more peaks
}

bool PitchDetector::similar(float a, float b)
{
  float q = a / b;
  
  return (q >= 0.85  &&  q <= 1.15);
}

bool PitchDetector::notTooDifferent(float a, float b)
{
  float q = a / b;
  
  return (q >= 0.7  &&  q <= 1.3);
}
