
/* Sound handler implementation

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <string.h>
#include <math.h>
#include "SoundHandler.h"
#include "../../common/include/sound_commands.h"

/*
 * Cosntructor
 */
SoundHandler::SoundHandler()
{
  memset(commandControl, 0, sizeof(CommandControl)); 
}

/*
 * Start recording
 */
void SoundHandler::StartRecording(MicGain gain, RecMode mode, int rate, void* buffer, int length)
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  
  StartRecordingCommand* sr = &command->startRecording;

  command->commandType = START_RECORDING; 
  
  sr->gain = (short)gain;
  sr->mode = (short)mode;
  sr->rate = rate;
  sr->buffer = buffer;
  sr->length = length;

  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
}

/*
 * Restart recording
 */
void SoundHandler::RestartRecording(void* buffer, int length)
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  
  RestartRecordingCommand* sr = &command->restartRecording;

  command->commandType = RESTART_RECORDING; 
  
  sr->buffer = buffer;
  sr->length = length;

  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
}

/*
 * Stop recording
 */
int SoundHandler::StopRecording()
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  
  command->commandType = STOP_RECORDING; 
  
  commandControl->return_data = -1;
  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
  
  while(commandControl->return_data == -1)
    swiDelay(1);
    
  return (commandControl->return_data);
}

/*
 * Pause recording
 */
int SoundHandler::PauseRecording()
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  
  command->commandType = PAUSE_RECORDING; 
  
  commandControl->return_data = -1;
  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
  
  while(commandControl->return_data == -1)
    swiDelay(1);
    
  return (commandControl->return_data);
}

/*
 * Play the requested sample
 */
void SoundHandler::PlaySample(float freq, void *sampleData, int lSampleData)
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  PlaySampleSoundCommand* ps = &command->playSample;

  command->commandType = PLAY_ONE_SHOT_SAMPLE; 
  ps->channel = 0;
  ps->frequency = (int)freq;
  ps->data = sampleData;
  ps->length = lSampleData;
  ps->volume = 127;

  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
}

/*
 * Play the requested loop sample
 */
void SoundHandler::PlayLoopSample(float freq, void *sampleData, int lSampleData)
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  PlaySampleSoundCommand* ps = &command->playSample;

  command->commandType = PLAY_LOOP_SAMPLE; 
  ps->channel = 0;
  ps->frequency = (int)(freq * lSampleData);
  ps->data = sampleData;
  ps->length = lSampleData;
  ps->volume = 127;

  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
}

/*
 * Stop play
 */
void SoundHandler::StopPlaySample()
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  StopPlaySoundCommand* sp = &command->stopPlay;
  
  command->commandType = STOP_PLAY; 
  sp->channel = 0;
  
  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
}

void SoundHandler::CommandPlayOneShotSample(int channel, int frequency, void* data, int length, int volume)
{
  SoundCommand* command = &commandControl->command[commandControl->currentCommand];
  PlaySampleSoundCommand* ps = &command->playSample;

  command->commandType = PLAY_ONE_SHOT_SAMPLE; 
  ps->channel = channel;
  ps->frequency = frequency;
  ps->data = data;
  ps->length = length;
  ps->volume = volume;

  commandControl->currentCommand++;
  commandControl->currentCommand &= MAX_COMMANDS-1;
}

