
/* Timers handler implementation
 
Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include "TimersHandler.h"

/* 
 * Constructor
 */
Timer::Timer(int timeout, bool bAutoRestart)
{
  m_timeout = timeout;
  m_bAutoRestart = bAutoRestart;
  
  Start();
}

/*
 * Check if elapsed
 */
bool Timer::IsElapsed()
{
  if (GetTimeCounter() < m_tctrTimeout)
    return (false);
    
  if (m_bAutoRestart)
    Start();
  
  return (true);
}

/*
 * Touch timer
 */
void Timer::Touch()
{
  m_tctrTimeout = 0;
}

/*
 * Restart timer
 */
void Timer::Restart()
{
  Start();
}

/*
 * Start the timer
 */
void Timer::Start()
{
  m_tctrTimeout = GetTimeCounter() + m_timeout;
}

/*
 * Get the time counter
 */
unsigned long Timer::GetTimeCounter()
{
  while (true)
  {
    unsigned long n = tctr;
    unsigned long m = tctr;
    
    if (n == m)
      return (n);
  }
}

/*
 * Create a new timer
 */
Timer* TimersHandler::CreateTimer(int timeout, bool bAutoRestart)
{
  return (new Timer(timeout, bAutoRestart));
}
