
/* Tuner commander implementation

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>
#include <string.h>

#include "sound_bmp_dat.h"
#include "LiteralHandler.h"
#define TUNERCOMMANDERDATA
#include "TunerCommander.h"

static Bitmap sndBmp;

/*
 * Constructor
 */
TunerCommander::TunerCommander(GraphicsHandler* gh, ToneCalculator *tc)
{
  GraphicsHandler::MakeBitmap(&sndBmp, sound_bmp_dat, true, 0x0F95);
  
  m_gh = gh;
  
  m_tc = tc;
  
  m_bh = new ButtonsHandler(gh);
  
  m_ch = new CombosHandler(gh, m_bh);
  
  m_gh->ClearScreen(BCK_COLOR);
  
  DisplayMainPanel();
  
  DisplayManualPanel();
  
  DisplayTarget();
  
  m_targetChanged = true;
  
  gh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_WEBSITE, -1, 190, 11, LOGO_COLOR, BCK_COLOR);
  
  m_bPenDown = false;
}

/*
 * Destructor
 */
TunerCommander::~TunerCommander()
{
  delete m_ch;
  delete m_bh;
}
  
/*
 * Handle commands
 */
TunerCommander::ExternalCommand TunerCommander::HandleCommands()
{
  /* Check for recording toggling */
  
  scanKeys();
  
  if ((keysHeld() & (KEY_R|KEY_L)) == (KEY_R|KEY_L))   // left + right
  {
    do
    {
      swiWaitForVBlank();
      
      scanKeys();
      
    } while ((keysHeld() & (KEY_R|KEY_L)) != 0);
    
    return (TOGGLE_RECORDING);
  }
  
  /* Handle touch commands */
  
  if(keysHeld() & KEY_TOUCH)
  {
    touchPosition touch = touchReadXY();
        
    if (touch.px != 0  &&  touch.py != 0)
    {
      if (!m_bPenDown)
      {
        m_bPenDown = true;
        
        /* Handle combos */
        
        Combo* cmb = m_ch->CheckComboPrevAt(touch.px, touch.py);
        
        if (cmb != 0)
          cmb->Prev();
        else
        {
          cmb = m_ch->CheckComboNextAt(touch.px, touch.py);
          
          if (cmb != 0)
            cmb->Next();
        }
        
        if (cmb != 0)
          HandleComboSelectedChanged(cmb);
               
        /* Handle buttons */
        
        if (cmb == 0)
        {
          Button* btn = m_bh->CheckButtonAt(touch.px, touch.py);
          
          if (btn != 0  &&  btn->GetId() == IDB_PLAY_TONE)
          {
            /* Start play */
            
            return (PLAY_TONE);
          }
        }
      }
    }
  }
  else
    m_bPenDown = false;
    
  /* Check if target is changed */
  
  if (m_targetChanged)
  {
    m_targetChanged = false;
    
    return (TARGET_CHANGED);
  }
  
  return (NONE);
}

/*
 * Check if working in autodetect mode
 */
bool TunerCommander::IsAutodetect()
{
  return (m_curNote == IDN_AUTO);
}

/*
 * Get Target note
 */
int TunerCommander::GetTargetNote()
{
  return ((m_curNote == IDN_AUTO) ? -1 : m_curNote);
}

/*
 * Get target octave
 */
int TunerCommander::GetTargetOctave()
{
  return ((m_curNote == IDN_AUTO) ? -1 : m_curOctave);
}

/*
 * Get target frequency (requested through external command)
 */
float TunerCommander::GetTargetFrequency()
{
  return ((m_curNote == IDN_AUTO) ? -1.0 : m_curFreq);
}


/*
 * Display main panel
 */
void TunerCommander::DisplayMainPanel()
{
  m_gh->DrawText(LIT_LBLINS, XLBL_INS, YLBL_INS, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_INS_COMBO, XCMB_INS, YCMB_INS, WCMB_ALL, HCMB_ALL, instruments, NODEP, IDS_MANUAL);
  
  m_gh->DrawText(LIT_LBLAFREQ, XLBL_AFREQ, YLBL_AFREQ, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_AFREQ_COMBO, XCMB_AFREQ, YCMB_AFREQ, WCMB_ALL, HCMB_ALL, Afreqs, NODEP, IDF_440);
}

/*
 * Remove main panel
 */
void TunerCommander::RemoveMainPanel()
{
  ClearTopPanel();
  
  m_ch->RemoveCombo(IDC_INS_COMBO);
  m_ch->RemoveCombo(IDC_AFREQ_COMBO);
}

/*
 * Display guitar panel
 */
void TunerCommander::DisplayGuitarPanel()
{
  m_gh->DrawText(LIT_LBLTMODE, XLBL_TMODE, YLBL_TMODE, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_TGMODE_COMBO, XCMB_TMODE, YCMB_TMODE, WCMB_ALL, HCMB_ALL, guiModes, NODEP, IDGM_STD);
  
  m_gh->DrawText(LIT_LBLSTRING, XLBL_STRING, YLBL_STRING, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_STRING_COMBO, XCMB_STRING, YCMB_STRING, WCMB_ALL, HCMB_ALL, guiStrings, IDGM_STD, IDGS_6);
  
  SetStringNoteAndOctave(IDS_GUITAR, IDC_TGMODE_COMBO, IDC_STRING_COMBO);
}

/*
 * Remove guitar panel
 */
void TunerCommander::RemoveGuitarPanel()
{
  ClearMidPanel();
  
  m_ch->RemoveCombo(IDC_TGMODE_COMBO);
  m_ch->RemoveCombo(IDC_STRING_COMBO);
}

/*
 * Display bass panel
 */
void TunerCommander::DisplayBassPanel()
{
  m_gh->DrawText(LIT_LBLTMODE, XLBL_TMODE, YLBL_TMODE, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_TBMODE_COMBO, XCMB_TMODE, YCMB_TMODE, WCMB_ALL, HCMB_ALL, bassModes, NODEP, IDBM_STD);
  
  m_gh->DrawText(LIT_LBLSTRING, XLBL_STRING, YLBL_STRING, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_STRING_COMBO, XCMB_STRING, YCMB_STRING, WCMB_ALL, HCMB_ALL, bassStrings, IDBM_STD, IDBS_4);
  
  SetStringNoteAndOctave(IDS_BASS, IDC_TBMODE_COMBO, IDC_STRING_COMBO);
}

/*
 * Remove bass panel
 */
void TunerCommander::RemoveBassPanel()
{
  ClearMidPanel();
  
  m_ch->RemoveCombo(IDC_TBMODE_COMBO);
  m_ch->RemoveCombo(IDC_STRING_COMBO);
}

/*
 * Display manual panel
 */
void TunerCommander::DisplayManualPanel()
{
  m_gh->DrawText(LIT_LBLNOTE, XLBL_NOTE, YLBL_NOTE, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_NOTE_COMBO, XCMB_NOTE, YCMB_NOTE, WCMB_ALL, HCMB_ALL, notes, NODEP, IDN_AUTO);
  
  m_gh->DrawText(LIT_LBLOCTAVE, XLBL_OCTAVE, YLBL_OCTAVE, LBL_COLOR, false, 0);
  
  m_ch->AddCombo(IDC_OCTAVE_COMBO, XCMB_OCTAVE, YCMB_OCTAVE, WCMB_ALL, HCMB_ALL, octaves, IDN_AUTO, IDO_AUTO);
  
  SetCurrentNote(IDN_AUTO);
  SetCurrentOctave(IDO_AUTO);
}

/*
 * Remove manual panel
 */
void TunerCommander::RemoveManualPanel()
{
  ClearMidPanel();
  
  m_ch->RemoveCombo(IDC_NOTE_COMBO);
  m_ch->RemoveCombo(IDC_OCTAVE_COMBO);
}

/*
 * Display target
 */
void TunerCommander::DisplayTarget()
{
  if (m_curNote == IDN_AUTO)
  {
    /* Autodetection */
    
    m_bh->RemoveButton(IDB_PLAY_TONE);
    
    ClearBottomPanel();
    
    m_gh->DrawText(LIT_LBLAUTODETECT, XLBL_TARGET, YLBL_TARGET, LBL_COLOR, true, BCK_COLOR);
  }
  else
  {
    /* Target defined */
    
    if (m_bh->GetButton(IDB_PLAY_TONE) == 0)
    {
      ClearBottomPanel();
      
      m_bh->AddButton(IDB_PLAY_TONE, XBTN_PTONE, YBTN_PTONE, WBTN_PTONE, HBTN_PTONE, &sndBmp);
    }
    
    m_curFreq = m_tc->getFreq(m_curNote, m_curOctave);
    
    m_gh->DrawText(LIT_LBLTARGET, XLBL_TARGET, YLBL_TARGET, LBL_COLOR, true, BCK_COLOR);
    
    char buff[40];
    
    sprintf(buff, LIT_LBLTARGETDATA, LiteralHandler::GetLit(NLIT_FIRSTNOTE+m_curNote), m_curOctave, m_curFreq);
    
    if (strlen(buff) > 16)
      buff[16] = '\0';
    
    m_gh->DrawText(buff, XLBL_TARGETDATA, YLBL_TARGETDATA, LBL_COLOR, true, BCK_COLOR);
  }
}

/*
 * Clear top panel
 */
void TunerCommander::ClearTopPanel()
{
  ClearPanel(TOP_PANEL_X, TOP_PANEL_Y, TOP_PANEL_W, TOP_PANEL_H);
}

/*
 * Clear mid panel
 */
void TunerCommander::ClearMidPanel()
{
  ClearPanel(MID_PANEL_X, MID_PANEL_Y, MID_PANEL_W, MID_PANEL_H);
}

/*
 * Clear bottom panel
 */
void TunerCommander::ClearBottomPanel()
{
  ClearPanel(BTN_PANEL_X, BTN_PANEL_Y, BTN_PANEL_W, BTN_PANEL_H);
}

/*
 * Clear panel
 */
void TunerCommander::ClearPanel(int x, int y, int w, int h)
{
  m_gh->DrawRectangle(x, y, x+w, y+h, BCK_COLOR, true);
}

/*
 * Handle selection changed on requested combo
 */
void TunerCommander::HandleComboSelectedChanged(Combo *cmb)
{
  switch (cmb->GetId())
  {
    case IDC_INS_COMBO:
      OnInstrumentSelected(cmb->GetCurElementId());
      break;
      
    case IDC_AFREQ_COMBO:
      OnAFreqSelected(cmb->GetCurElementId());
      break;
      
    case IDC_NOTE_COMBO:
      OnNoteSelected(cmb->GetCurElementId());
      break;
      
    case IDC_OCTAVE_COMBO:
      OnOctaveSelected(cmb->GetCurElementId());
      break;
      
    case IDC_TGMODE_COMBO:
      OnGuitarTuningModeSelected(cmb->GetCurElementId());
      break;
      
    case IDC_TBMODE_COMBO:
      OnBassTuningModeSelected(cmb->GetCurElementId());
      break;
      
    case IDC_STRING_COMBO:
      OnStringSelected(cmb->GetCurElementId());
      break;
  }
}

/*
 * Handle instrument selection
 */
void TunerCommander::OnInstrumentSelected(int id)
{
  RemoveGuitarPanel();
  RemoveBassPanel();
  RemoveManualPanel();
  
  switch(id)
  {
    case IDS_GUITAR:
      DisplayGuitarPanel();
      break;
      
    case IDS_BASS:
      DisplayBassPanel();
      break;
      
    case IDS_MANUAL:
      DisplayManualPanel();
      break;
  }
  
  DisplayTarget();
}

/*
 * Handle A frequency selection
 */
void TunerCommander::OnAFreqSelected(int id)
{
  m_tc->setNewFreqOf_A((float)id);
     
  m_targetChanged = true;

  DisplayTarget();
}

/*
 * Handle note selection
 */
void TunerCommander::OnNoteSelected(int id)
{
  SetCurrentNote(id);
  
  if (id == IDN_AUTO)
  {
    m_ch->ReloadCombo(IDC_OCTAVE_COMBO, octaves, IDN_AUTO, IDO_AUTO);
    
    SetCurrentOctave(IDO_AUTO);
  }
  else if (m_curOctave == IDO_AUTO)
  {
    m_ch->ReloadCombo(IDC_OCTAVE_COMBO, octaves, NODEP, IDO_2);
    
    SetCurrentOctave(IDO_2);
  }
  
  DisplayTarget();
}

/*
 * Handle octave selection
 */
void TunerCommander::OnOctaveSelected(int id)
{
  SetCurrentOctave(id);
  
  DisplayTarget();
}

/*
 * Handle guitar tuning mode selection
 */
void TunerCommander::OnGuitarTuningModeSelected(int id)
{
  m_ch->ReloadCombo(IDC_STRING_COMBO, guiStrings, id, IDGS_6);
  
  SetStringNoteAndOctave(IDS_GUITAR, IDC_TGMODE_COMBO, IDC_STRING_COMBO);
  
  DisplayTarget();
}

/*
 * Handle bass tuning mode selection
 */
void TunerCommander::OnBassTuningModeSelected(int id)
{
  m_ch->ReloadCombo(IDC_STRING_COMBO, bassStrings, id, IDBS_4);
  
  SetStringNoteAndOctave(IDS_BASS, IDC_TBMODE_COMBO, IDC_STRING_COMBO);
  
  DisplayTarget();
}

/*
 * Handle string selection
 */
void TunerCommander::OnStringSelected(int id)
{
  int ins = m_ch->GetCombo(IDC_INS_COMBO)->GetCurElementId();
  
  SetStringNoteAndOctave(ins, ((ins == IDS_GUITAR) ? IDC_TGMODE_COMBO : IDC_TBMODE_COMBO), IDC_STRING_COMBO);
  
  DisplayTarget();
}

/*
 * Set current note and octave, starting from current settings
 */
void TunerCommander::SetStringNoteAndOctave(int idIns, int idcTuningMode, int idcString)
{  
  int idTuningMode = m_ch->GetCombo(idcTuningMode)->GetCurElementId();
  int idString = m_ch->GetCombo(idcString)->GetCurElementId();
  
  int i=0;
  
  while (true)
  {
    if (strings[i].idIns == -1)
      return;   // wrong cfg
      
    if (strings[i].idIns == idIns  &&  strings[i].idTuningMode == idTuningMode  &&  strings[i].idString == idString)
    {
      SetCurrentNote(strings[i].idNote);
      SetCurrentOctave(strings[i].idOctave);
      
      return;
    }
    
    i++;
  }
}

/*
 * Set current note
 */
void TunerCommander::SetCurrentNote(int idNote)
{
  m_curNote = (idNote == IDN_AUTO) ? IDN_AUTO : (idNote - IDN_C);
  
  if (idNote == IDN_AUTO)
    SetCurrentOctave(IDO_AUTO);
    
  m_targetChanged = true;
}

/*
 * Set current octave
 */
void TunerCommander::SetCurrentOctave(int idOctave)
{
  m_curOctave = (idOctave == IDO_AUTO) ? IDO_AUTO : (idOctave - IDO_0);
  
  m_targetChanged = true;
}
