
/*
  SoundCommands.h - Structures and functions to allow the ARM9 to send commands to the ARM7.
*/

#ifndef SOUND_COMMANDS_H
#define SOUND_COMMANDS_H

/* 
 * Input signal gain level 
 */
#define GAIN_20           0
#define GAIN_40           1
#define GAIN_80           2
#define GAIN_160          3

/* 
 * Sampling mode 
 */
#define MODE_8BIT         0
#define MODE_12BIT        1

/* 
 * Enumeration of commands that the ARM9 can send to the ARM7 
 */
enum SoundCommandType
{
  PLAY_ONE_SHOT_SAMPLE,
  PLAY_LOOP_SAMPLE,
  STOP_PLAY,
  START_RECORDING,
  STOP_RECORDING,
  PAUSE_RECORDING,
  RESTART_RECORDING
};

/* 
 * Command parameters for playing a sound sample 
 */
struct PlaySampleSoundCommand
{
  int channel;
  int frequency;
  void* data;
  int length;
  int volume;
};

/* 
 * Command parameters for playing a sound sample 
 */
struct StopPlaySoundCommand
{
  int channel;
};

/* 
 * Command parameters for starting to record from the microphone 
 */
struct StartRecordingCommand
{
  short gain;
  short mode;
  int rate;
  void* buffer;  
  int length;
};

/* 
 * Command parameters for restarting to record from the microphone 
 */
struct RestartRecordingCommand
{
  void* buffer;  
  int length;
};

/* 
 * The ARM9 fills out values in this structure to tell the ARM7 what to do. 
 */
struct SoundCommand 
{
  SoundCommandType commandType;
  union 
  {
    void* data;  
    PlaySampleSoundCommand playSample;    
    StopPlaySoundCommand stopPlay;
    StartRecordingCommand  startRecording;
    RestartRecordingCommand  restartRecording;
  };
};

/* Maximum number of commands */

#define MAX_COMMANDS 20

/* A structure shared between the ARM7 and ARM9. The ARM9
   places commands here and the ARM7 reads and acts upon them.
*/

struct CommandControl 
{
  SoundCommand command[MAX_COMMANDS];
  int currentCommand;
  int return_data;
};

/* Address of the shared CommandControl structure */

#define commandControl ((CommandControl*)((uint32)(IPC) + sizeof(TransferRegion)))

#endif
